// LICENSE : MIT
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.separateByAvailability = exports.findFiles = exports.pathsToGlobPatterns = void 0;
const path_to_glob_pattern_1 = require("path-to-glob-pattern");
const glob_1 = __importDefault(require("glob"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)("textlint:find-util");
const DEFAULT_IGNORE_PATTERNS = Object.freeze(["**/.git/**", "**/node_modules/**"]);
const mapGitIgnorePatternTo = (base) => (ignore) => {
    if (ignore.startsWith("!")) {
        return `!${path_1.default.posix.join(base, ignore.slice(1))}`;
    }
    return path_1.default.posix.join(base, ignore);
};
const isFile = (filePath) => {
    try {
        return fs_1.default.statSync(filePath).isFile();
    }
    catch (error) {
        return false;
    }
};
/**
 * filter files by config
 * @param {string[]} patterns glob patterns
 * @param {{extensions?: string[], cwd?: string }} options
 */
function pathsToGlobPatterns(patterns, options = {}) {
    const processPatterns = (0, path_to_glob_pattern_1.pathToGlobPattern)({
        extensions: options.extensions || [],
        cwd: options.cwd || process.cwd()
    });
    return patterns.map(processPatterns);
}
exports.pathsToGlobPatterns = pathsToGlobPatterns;
/**
 * found files by glob pattern
 * @param {string[]} patterns
 * @param {FindFilesOptions} options
 * @returns {string[]} file path list
 */
function findFiles(patterns, options = {}) {
    const cwd = options.cwd || process.cwd();
    const ignoredPatterns = [];
    ignoredPatterns.push(...DEFAULT_IGNORE_PATTERNS);
    if (options.ignoreFilePath) {
        const baseDir = path_1.default.resolve(cwd, path_1.default.dirname(options.ignoreFilePath));
        const normalizeIgnoreFilePath = path_1.default.resolve(cwd, options.ignoreFilePath);
        debug("findFiles ignore baseDir: %s, normalizeIgnoreFilePath: %s", baseDir, normalizeIgnoreFilePath);
        if (fs_1.default.existsSync(normalizeIgnoreFilePath)) {
            const ignored = fs_1.default
                .readFileSync(normalizeIgnoreFilePath, "utf-8")
                .split(/\r?\n/)
                .filter((line) => !/^\s*$/.test(line) && !/^\s*#/.test(line))
                .map(mapGitIgnorePatternTo(baseDir));
            debug("ignored: %o", ignored);
            ignoredPatterns.push(...ignored);
        }
    }
    debug("search patterns: %o", patterns);
    debug("search ignore patterns: %o", ignoredPatterns);
    const files = [];
    const addFile = (filePath) => {
        if (files.indexOf(filePath) === -1) {
            files.push(filePath);
        }
    };
    patterns.forEach((pattern) => {
        const file = path_1.default.resolve(cwd, pattern);
        if (isFile(file)) {
            addFile(fs_1.default.realpathSync(file));
        }
        else {
            glob_1.default.sync(pattern, {
                cwd,
                absolute: true,
                nodir: true,
                ignore: ignoredPatterns
            }).forEach((filePath) => {
                // workaround for windows
                // https://github.com/isaacs/node-glob/issues/74#issuecomment-31548810
                addFile(path_1.default.resolve(filePath));
            });
        }
    });
    return files;
}
exports.findFiles = findFiles;
/**
 * classify files by extname
 * unAvailableFilePath will be used for warning.
 * If The user can use glob pattern like `src/*.js` as arguments, It will be expanded by shell.
 * pathsToGlobPatterns not modified that pattern.
 * So, unAvailableFilePath should be ignored silently.
 * @param {string[]} files
 * @param {{extensions?: string[]}} [options]
 * @returns {{availableFiles: string[], unAvailableFiles: string[]}}
 */
function separateByAvailability(files, options = {}) {
    const extensions = options.extensions || [];
    const availableFiles = [];
    const unAvailableFiles = [];
    files.forEach((filePath) => {
        const extname = path_1.default.extname(filePath) || path_1.default.basename(filePath);
        if (extensions.indexOf(extname) === -1) {
            unAvailableFiles.push(filePath);
        }
        else {
            availableFiles.push(filePath);
        }
    });
    return {
        availableFiles,
        unAvailableFiles
    };
}
exports.separateByAvailability = separateByAvailability;
//# sourceMappingURL=find-util.js.map